@@ -16,10 +16,11 @@
 ##############################################################################
 
 import textwrap
+import argparse
 
 from PyQt4 import QtCore, QtGui
 
-__VERSION__ = "0.0.3"
+__VERSION__ = "0.0.4"
 
 def comb(str1, str2):
     return str1 + str2
@@ -63,7 +64,8 @@ _OR_MORE = {
 
 class ArgparseUi(QtGui.QDialog):
     def __init__(self, parser, use_scrollbars=False, remove_defaults_from_helptext=False,
-                 helptext_default=' [default=%(default)s]', use_save_load_button=False, window_title="Make your choice", parent=None):
+                 helptext_default=' [default=%(default)s]', use_save_load_button=False, window_title="Make your choice", 
+                 left_label_alignment=None, ok_button_handler=None, cancel_button_handler=None, parent=None):
         super(ArgparseUi, self).__init__(parent)
         self.setWindowTitle(window_title)
         self.parser = parser
@@ -71,6 +73,8 @@ class ArgparseUi(QtGui.QDialog):
         self.remove_defaults_from_helptext = remove_defaults_from_helptext
         self.helptext_default = helptext_default
         self.use_save_load_button = use_save_load_button
+        self.ok_button_handler = ok_button_handler # function that takes two options: the ArgparseUi instance and "parsed options"
+        self.cancel_button_handler= cancel_button_handler # function that takes one option: the ArgparseUi instance
         self.commandLineArgumentCreators = []
         self.filename = None
         self.destToWidget = {}
@@ -84,6 +88,8 @@ class ArgparseUi(QtGui.QDialog):
         
         self.options = QtGui.QWidget(self)
         self.optionsLayout = QtGui.QFormLayout(self.options)
+        if left_label_alignment is not None:
+            self.optionsLayout.setLabelAlignment(QtCore.Qt.AlignLeft if left_label_alignment else QtCore.Qt.AlignRight)
         self.options.setLayout(self.optionsLayout)
 
         self.buttons = QtGui.QWidget(self)
@@ -149,8 +155,6 @@ class ArgparseUi(QtGui.QDialog):
         and an ok and cancel button
         """
         self.addDescription()
-        
-        import argparse    
         self.actionLookupTable = { 
                     type(argparse._HelpAction(None)) : self.makeHelpActionEntry,
                     type(argparse._VersionAction(None)) : self.makeHelpActionEntry,                   
@@ -349,7 +353,7 @@ class ArgparseUi(QtGui.QDialog):
         while building the ui)
         """
         def to_command_line():
-            if type(include_widget) == type(QtGui.QCheckBox()):
+            if type(include_widget) == QtGui.QCheckBox:
                 checked = include_widget.isChecked()
             else:
                 checked = True
@@ -377,7 +381,9 @@ class ArgparseUi(QtGui.QDialog):
                 
             if optional:
                 include = QtGui.QCheckBox(comb(helpstring, typehelp), self.options)
-                self.disableOnClick(combobox)(False)                
+                enabled = a.default is not None
+                include.setChecked(enabled)
+                self.disableOnClick(combobox)(enabled)                
                 include.clicked.connect(self.disableOnClick(combobox))                        
                 self.registerDisplayStateInfo(a.dest, [include])
             else:
@@ -396,7 +402,9 @@ class ArgparseUi(QtGui.QDialog):
             
             if optional:
                 include = QtGui.QCheckBox(comb(helpstring, typehelp), self.options)
-                self.disableOnClick(lineedit)(False)
+                enabled = a.default is not None
+                include.setChecked(enabled)
+                self.disableOnClick(lineedit)(enabled)
                 include.clicked.connect(self.disableOnClick(lineedit))        
                 self.registerDisplayStateInfo(a.dest, [include])
             else:
@@ -413,12 +421,12 @@ class ArgparseUi(QtGui.QDialog):
         while building the ui)
         """
         def to_command_line():
-            if type(include_widget) == type(QtGui.QCheckBox()):
+            if type(include_widget) == QtGui.QCheckBox:
                 checked = include_widget.isChecked()
             else:
                 checked = True
             if checked:
-                if type(value_widget) == type(QtGui.QLineEdit()):
+                if type(value_widget) == QtGui.QLineEdit:
                     if argument.option_strings:
                         if argument.nargs == "1" or argument.nargs is None:
                           return ["{0}".format(argument.option_strings[0]),  
@@ -429,7 +437,7 @@ class ArgparseUi(QtGui.QDialog):
                           return cmd
                     else:
                         return ["{0}".format(value_widget.text())]
-                elif type(value_widget) == type(QtGui.QComboBox()):
+                elif type(value_widget) == QtGui.QComboBox:
                     if argument.option_strings:
                         return ["{0}".format(argument.option_strings[0]), 
                                      "{0}".format(value_widget.currentText())] 
@@ -452,7 +460,9 @@ class ArgparseUi(QtGui.QDialog):
         spinbox.setValue(a.nargs)
         if optional:
             include = QtGui.QCheckBox(helpstring, self.options)                                
-            self.disableOnClick(spinbox)(False)
+            enabled = a.default is not None
+            include.setChecked(enabled)
+            self.disableOnClick(spinbox)(enabled)
             include.clicked.connect(self.disableOnClick(spinbox))                
             self.registerDisplayStateInfo(a.dest, [include])
         else:
@@ -470,12 +480,12 @@ class ArgparseUi(QtGui.QDialog):
         """
 
         def to_command_line():
-            if type(include_widget) == type(QtGui.QCheckBox()):
+            if type(include_widget) == QtGui.QCheckBox:
                 checked = include_widget.isChecked()
             else:
                 checked = True
             if checked:
-                if type(value_widget) == type(QtGui.QSpinBox()):
+                if type(value_widget) == QtGui.QSpinBox:
                     if argument.option_strings:
                         return [("{0}".format(argument.option_strings[0]))]*value_widget.value()
                     else:
@@ -533,7 +543,9 @@ class ArgparseUi(QtGui.QDialog):
             
         if optional:
             include = QtGui.QCheckBox(helpstring, self.options)                                
-            self.disableOnClick(tablewidget)(False)
+            enabled = a.default is not None
+            include.setChecked(enabled)           
+            self.disableOnClick(tablewidget)(enabled)
             include.clicked.connect(self.disableOnClick(tablewidget))                
             self.registerDisplayStateInfo(a.dest, [include])
         else:
@@ -551,12 +563,12 @@ class ArgparseUi(QtGui.QDialog):
         """
         def to_command_line():
             data = []
-            if type(include_widget) == type(QtGui.QCheckBox()):
+            if type(include_widget) == QtGui.QCheckBox:
                 checked = include_widget.isChecked()
             else:
                 checked = True
             if checked:
-                if type(tablewidget) == type(QtGui.QTableWidget()):
+                if type(tablewidget) == QtGui.QTableWidget:
                     if argument.option_strings:
                         for c in range(tablewidget.columnCount()):
                           cellText = "{0}".format(tablewidget.item(0, c).text() if tablewidget.item(0, c) else "")
@@ -614,7 +626,10 @@ class ArgparseUi(QtGui.QDialog):
         """
         validate, offensive_options = self.validateMutualExclusiveOptions()
         if validate:
-            self.accept()
+            if self.ok_button_handler is None:
+                self.accept()
+            else:
+                self.ok_button_handler(self)
         else:
             mutexes = "\n".join([",".join(o) for o in offensive_options])
             QtGui.QMessageBox.question(self, 'Validation error', "The following options are mutually exclusive:\n{0}".format(mutexes), QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
@@ -623,7 +638,10 @@ class ArgparseUi(QtGui.QDialog):
         """
         handle cancel button pressed
         """
-        self.reject()
+        if self.cancel_button_handler is None:
+            self.reject()
+        else:
+            self.cancel_button_handler(self)
 
     def resetAllWidgets(self, argparser):
         helper = argparse.ArgumentParser(add_help=False, parents=[self.parser], fromfile_prefix_chars='@')
@@ -638,6 +656,9 @@ class ArgparseUi(QtGui.QDialog):
             if type(w) == QtGui.QCheckBox:
               w.setChecked(False)
               w.clicked.emit(False)
+            elif type(w) == QtGui.QTableWidget:
+              w.setColumnCount(1)
+              w.setItem(0, 0, QtGui.QTableWidgetItem(""))
 
     def onLoad(self):
         """
@@ -664,9 +685,13 @@ class ArgparseUi(QtGui.QDialog):
           for w in self.destToWidget[a.dest]:
             data = result.__getattribute__(a.dest)
             if type(w) == QtGui.QCheckBox:
-              if data:
-                w.setChecked(True)
-                w.clicked.emit(True)
+              if data is not None:
+                if type(data) == bool:
+                    w.setChecked(data)
+                    w.clicked.emit(data)
+                else:
+                    w.setChecked(True)
+                    w.clicked.emit(True)
             elif type(w) == QtGui.QLineEdit:
               if data is not None:
                 if type(data) == type([]):
@@ -676,12 +701,15 @@ class ArgparseUi(QtGui.QDialog):
             elif type(w) == QtGui.QSpinBox:
               if data is not None:
                 w.setValue(int("{0}".format(data)))
+            elif type(w) == QtGui.QComboBox:
+              if data is not None:
+                w.setCurrentIndex(w.findText("{0}".format(data)))                
             elif type(w) == QtGui.QTableWidget:
               #clear existing defaults first
               w.setColumnCount(1)
               w.setItem(0, 0, QtGui.QTableWidgetItem(""))
               cnt = w.columnCount()
-              if data:
+              if data is not None:
                 needed = len(data)
                 if needed >= cnt:
                   for i in range(needed-cnt+1):
@@ -725,7 +753,6 @@ class ArgparseUi(QtGui.QDialog):
         return self.parser.parse_args(cmdline)
             
 if __name__ == "__main__":
-    import argparse
     import sys
     from PyQt4 import QtGui
     
@@ -744,7 +771,7 @@ if __name__ == "__main__":
     parser.add_argument('--foo', type=int, nargs='+')
     parser.add_argument('--bar', type=int, nargs=2, metavar=('bar', 'baz'))
     app = QtGui.QApplication(sys.argv)
-    a = ArgparseUi(parser, use_scrollbars=True, use_save_load_button=True)
+    a = ArgparseUi(parser, left_label_alignment=True, use_scrollbars=True, use_save_load_button=True)
     a.show()
     app.exec_()
     print ("Ok" if a.result() == 1 else "Cancel")
